# Generated by Django 5.0.6 on 2024-08-18 05:49

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0034_alter_archiveresult_old_id_alter_archiveresult_uuid'),
    ]

    operations = [
        migrations.RenameField(
            model_name='archiveresult',
            old_name='uuid',
            new_name='id',
        ),
    ]
