# Generated by Django 3.0.7 on 2020-07-13 15:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_auto_20200630_1034'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshot',
            name='timestamp',
            field=models.CharField(db_index=True, default=None, max_length=32, unique=True),
            preserve_default=False,
        ),
    ]
