# Generated by Django 5.0.6 on 2024-08-20 02:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0053_remove_snapshottag_snapshot_old'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshot',
            name='timestamp',
            field=models.CharField(db_index=True, editable=False, max_length=32, unique=True),
        ),
    ]
