from typing import Any

from django.utils.functional import cached_property

class BaseSpatialOperations:
    postgis: bool
    spatialite: bool
    mysql: bool
    oracle: bool
    spatial_version: Any
    select: str
    @cached_property
    def select_extent(self) -> str: ...
    geography: bool
    geometry: bool
    disallowed_aggregates: Any
    geom_func_prefix: str
    function_names: Any
    unsupported_functions: set[str]
    from_text: bool
    def convert_extent(self, box: Any, srid: Any) -> Any: ...
    def convert_extent3d(self, box: Any, srid: Any) -> Any: ...
    def geo_quote_name(self, name: Any) -> Any: ...
    def geo_db_type(self, f: Any) -> Any: ...
    def get_distance(self, f: Any, value: Any, lookup_type: Any) -> Any: ...
    def get_geom_placeholder(self, f: Any, value: Any, compiler: Any) -> Any: ...
    def check_expression_support(self, expression: Any) -> None: ...
    def spatial_aggregate_name(self, agg_name: Any) -> Any: ...
    def spatial_function_name(self, func_name: Any) -> Any: ...
    def geometry_columns(self) -> Any: ...
    def spatial_ref_sys(self) -> Any: ...
    distance_expr_for_lookup: Any
    def get_db_converters(self, expression: Any) -> Any: ...
    def get_geometry_converter(self, expression: Any) -> Any: ...
    def get_area_att_for_field(self, field: Any) -> Any: ...
    def get_distance_att_for_field(self, field: Any) -> Any: ...
