# Generated by Django 5.1 on 2024-09-05 00:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0071_remove_archiveresult_old_id_remove_snapshot_old_id_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='snapshot',
            old_name='added',
            new_name='bookmarked_at',
        ),
        migrations.RenameField(
            model_name='snapshot',
            old_name='updated',
            new_name='downloaded_at',
        ),
    ]
