# Generated by Django 5.1.1 on 2024-09-10 05:05

from django.db import migrations


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0074_alter_snapshot_downloaded_at'),
    ]

    operations = [
        migrations.CreateModel(
            name='SinglefileResult',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.archiveresult',),
        ),
    ]
