# Generated by Django 5.0.6 on 2024-08-20 03:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0060_alter_tag_id'),
    ]

    operations = [
        migrations.RenameField(
            model_name='snapshottag',
            old_name='tag',
            new_name='old_tag',
        ),
        migrations.AlterUniqueTogether(
            name='snapshottag',
            unique_together={('snapshot', 'old_tag')},
        ),
    ]
