# Generated by Django 5.0.6 on 2024-08-20 03:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0061_rename_tag_snapshottag_old_tag_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshottag',
            name='old_tag',
            field=models.ForeignKey(db_column='old_tag_id', on_delete=django.db.models.deletion.CASCADE, to='core.tag'),
        ),
    ]
