# Generated by Django 5.0.6 on 2024-08-19 23:01

import django.db.models.deletion
import uuid
from django.db import migrations, models



class Migration(migrations.Migration):

    dependencies = [
        ('core', '0043_alter_archiveresult_snapshot_alter_snapshot_id_and_more'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            database_operations=[
                # No-op, SnapshotTag model already exists in DB
            ],
            state_operations=[
                migrations.CreateModel(
                    name='SnapshotTag',
                    fields=[
                        ('id', models.AutoField(primary_key=True, serialize=False)),
                        ('snapshot', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.snapshot')),
                        ('tag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.tag')),
                    ],
                    options={
                        'db_table': 'core_snapshot_tags',
                        'unique_together': {('snapshot', 'tag')},
                    },
                ),
                migrations.AlterField(
                    model_name='snapshot',
                    name='tags',
                    field=models.ManyToManyField(blank=True, related_name='snapshot_set', through='core.SnapshotTag', to='core.tag'),
                ),
            ],
        ),
    ]
