from typing import Any

from django.db.models import Prefetch
from django.db.models.query import QuerySet

class GenericPrefetch(Prefetch):
    def __init__(self, lookup: str, querysets: list[QuerySet], to_attr: str | None = ...) -> None: ...
    def __getstate__(self) -> dict[str, Any]: ...
    def get_current_querysets(self, level: int) -> list[QuerySet] | None: ...
