# Generated by Django 5.0.6 on 2024-08-20 03:52

import core.models
import django.db.models.deletion
import uuid
import random
from django.db import migrations, models

def rand_int_id():
    return random.getrandbits(32)

class Migration(migrations.Migration):

    dependencies = [
        ('core', '0065_remove_snapshottag_old_tag'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshottag',
            name='tag',
            field=models.ForeignKey(db_column='tag_id', on_delete=django.db.models.deletion.CASCADE, to='core.tag', to_field='id'),
        ),
        migrations.AlterField(
            model_name='tag',
            name='id',
            field=models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True),
        ),
        migrations.AlterField(
            model_name='tag',
            name='old_id',
            field=models.BigIntegerField(default=rand_int_id, serialize=False, unique=True, verbose_name='Old ID'),
        ),
    ]
