# Generated by Django 3.1.3 on 2021-02-18 07:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0012_auto_20210216_1425'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshot',
            name='title',
            field=models.CharField(blank=True, db_index=True, max_length=256, null=True),
        ),
    ]
