# Generated by Django 5.1 on 2024-08-20 22:40

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0004_alter_apitoken_id_alter_apitoken_uuid'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='apitoken',
            name='uuid',
        ),
        migrations.RemoveField(
            model_name='outboundwebhook',
            name='id',
        ),
    ]
