# Generated by Django 5.0.6 on 2024-08-20 07:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0067_alter_snapshottag_tag'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='archiveresult',
            options={'verbose_name': 'Archive Result', 'verbose_name_plural': 'Archive Results Log'},
        ),
    ]
