# Generated by Django 5.0.6 on 2024-08-18 06:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0041_alter_archiveresult_snapshot_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='archiveresult',
            name='snapshot_old',
        ),
    ]
