# Generated by Django 5.1 on 2024-09-05 00:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0072_rename_added_snapshot_bookmarked_at_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='archiveresult',
            old_name='created',
            new_name='created_at',
        ),
        migrations.RenameField(
            model_name='archiveresult',
            old_name='modified',
            new_name='modified_at',
        ),
        migrations.RenameField(
            model_name='snapshot',
            old_name='created',
            new_name='created_at',
        ),
        migrations.RenameField(
            model_name='snapshot',
            old_name='modified',
            new_name='modified_at',
        ),
        migrations.RenameField(
            model_name='tag',
            old_name='created',
            new_name='created_at',
        ),
        migrations.RenameField(
            model_name='tag',
            old_name='modified',
            new_name='modified_at',
        ),
    ]
