# Generated by Django 5.0.6 on 2024-08-20 02:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0052_alter_snapshottag_unique_together_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='snapshottag',
            name='snapshot_old',
        ),
    ]
