# Generated by Django 5.0.6 on 2024-08-18 06:50

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0040_archiveresult_snapshot'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archiveresult',
            name='snapshot',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.snapshot', to_field='id'),
        ),
        migrations.AlterField(
            model_name='archiveresult',
            name='snapshot_old',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='archiveresults_old', to='core.snapshot'),
        ),
    ]
