# Generated by Django 5.0.6 on 2024-08-18 06:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0037_rename_id_snapshot_old_id'),
    ]

    operations = [
        migrations.RenameField(
            model_name='snapshot',
            old_name='uuid',
            new_name='id',
        ),
    ]
