# Generated by Django 5.0.6 on 2024-08-18 05:59

import core.models
import uuid
import random
from django.db import migrations, models


def rand_int_id():
    return random.getrandbits(32)

class Migration(migrations.Migration):

    dependencies = [
        ('core', '0035_remove_archiveresult_uuid_archiveresult_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archiveresult',
            name='id',
            field=models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False, unique=True, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='archiveresult',
            name='old_id',
            field=models.BigIntegerField(default=rand_int_id, serialize=False, verbose_name='Old ID'),
        ),
    ]
