# Generated by Django 5.1 on 2024-09-05 00:26

import abid_utils.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0008_alter_apitoken_created_alter_apitoken_created_by_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='apitoken',
            old_name='created',
            new_name='created_at',
        ),
        migrations.RenameField(
            model_name='apitoken',
            old_name='modified',
            new_name='modified_at',
        ),
        migrations.RenameField(
            model_name='outboundwebhook',
            old_name='modified',
            new_name='modified_at',
        ),
        migrations.AddField(
            model_name='outboundwebhook',
            name='created_at',
            field=abid_utils.models.AutoDateTimeField(db_index=True, default=None),
        ),
        migrations.AlterField(
            model_name='outboundwebhook',
            name='created',
            field=models.DateTimeField(auto_now_add=True, help_text='When the webhook was created.', verbose_name='created'),
        ),
    ]
