# Generated by Django 5.0.6 on 2024-08-20 02:37

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0051_snapshottag_snapshot_alter_snapshottag_snapshot_old'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='snapshottag',
            unique_together=set(),
        ),
        migrations.AlterField(
            model_name='snapshottag',
            name='snapshot',
            field=models.ForeignKey(db_column='snapshot_id', on_delete=django.db.models.deletion.CASCADE, to='core.snapshot'),
        ),
        migrations.AlterUniqueTogether(
            name='snapshottag',
            unique_together={('snapshot', 'tag')},
        ),
    ]
