# Generated by Django 5.0.6 on 2024-08-20 02:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0048_alter_archiveresult_snapshot_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='snapshottag',
            old_name='snapshot',
            new_name='snapshot_old',
        ),
        migrations.AlterUniqueTogether(
            name='snapshottag',
            unique_together={('snapshot_old', 'tag')},
        ),
    ]
