# Generated by Django 5.0.6 on 2024-05-13 10:56

import charidfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0022_auto_20231023_2008'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='archiveresult',
            options={'verbose_name': 'Result'},
        ),
        migrations.AddField(
            model_name='archiveresult',
            name='abid',
            field=charidfield.fields.CharIDField(blank=True, db_index=True, default=None, help_text='ABID-format identifier for this entity (e.g. snp_01BJQMF54D093DXEAWZ6JYRPAQ)', max_length=30, null=True, prefix='res_', unique=True),
        ),
        migrations.AddField(
            model_name='snapshot',
            name='abid',
            field=charidfield.fields.CharIDField(blank=True, db_index=True, default=None, help_text='ABID-format identifier for this entity (e.g. snp_01BJQMF54D093DXEAWZ6JYRPAQ)', max_length=30, null=True, prefix='snp_', unique=True),
        ),
        migrations.AddField(
            model_name='snapshot',
            name='uuid',
            field=models.UUIDField(blank=True, null=True, unique=True),
        ),
        migrations.AddField(
            model_name='tag',
            name='abid',
            field=charidfield.fields.CharIDField(blank=True, db_index=True, default=None, help_text='ABID-format identifier for this entity (e.g. snp_01BJQMF54D093DXEAWZ6JYRPAQ)', max_length=30, null=True, prefix='tag_', unique=True),
        ),
        migrations.AlterField(
            model_name='archiveresult',
            name='extractor',
            field=models.CharField(choices=(
                ('htmltotext', 'htmltotext'),
                ('git', 'git'),
                ('singlefile', 'singlefile'),
                ('media', 'media'),
                ('archive_org', 'archive_org'),
                ('readability', 'readability'),
                ('mercury', 'mercury'),
                ('favicon', 'favicon'),
                ('pdf', 'pdf'),
                ('headers', 'headers'),
                ('screenshot', 'screenshot'),
                ('dom', 'dom'),
                ('title', 'title'),
                ('wget', 'wget'),
            ), max_length=32),
        ),
    ]
