from django.core.exceptions import SuspiciousOperation

class DisallowedModelAdminLookup(SuspiciousOperation): ...
class DisallowedModelAdminToField(SuspiciousOperation): ...
class AlreadyRegistered(Exception): ...
class NotRegistered(Exception): ...
