# Generated by Django 5.0.6 on 2024-08-18 05:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0029_alter_archiveresult_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archiveresult',
            name='uuid',
            field=models.UUIDField(unique=True),
        ),
    ]
