# Generated by Django 5.1 on 2024-09-05 01:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0073_rename_created_archiveresult_created_at_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshot',
            name='downloaded_at',
            field=models.DateTimeField(blank=True, db_index=True, default=None, editable=False, null=True),
        ),
    ]
