# Generated by Django 5.1 on 2024-08-28 09:40

import abid_utils.models
import django.utils.timezone
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0068_alter_archiveresult_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archiveresult',
            name='created',
            field=abid_utils.models.AutoDateTimeField(db_index=True, default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='added',
            field=abid_utils.models.AutoDateTimeField(db_index=True, default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='created',
            field=abid_utils.models.AutoDateTimeField(db_index=True, default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='tag',
            name='created',
            field=abid_utils.models.AutoDateTimeField(db_index=True, default=django.utils.timezone.now),
        ),
    ]
