# Generated by Django 5.0.6 on 2024-08-20 03:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0056_remove_tag_uuid'),
    ]

    operations = [
        migrations.RenameField(
            model_name='tag',
            old_name='id',
            new_name='old_id',
        ),
    ]
