# Generated by Django 5.0.6 on 2024-08-20 03:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0055_alter_tag_slug'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tag',
            name='uuid',
        ),
    ]
