from typing import Any

class OGRGeomType:
    wkb25bit: int
    num: int
    def __init__(self, type_input: Any) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def django(self) -> str: ...
    def to_multi(self) -> None: ...
