# Generated by Django 5.1.1 on 2024-10-03 07:25

import archivebox.abid_utils.models
import charidfield.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("machine", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="machine",
            name="stats",
            field=models.JSONField(default=dict),
        ),
        migrations.CreateModel(
            name="InstalledBinary",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=None,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                        unique=True,
                        verbose_name="ID",
                    ),
                ),
                (
                    "abid",
                    charidfield.fields.CharIDField(
                        blank=True,
                        db_index=True,
                        default=None,
                        help_text="ABID-format identifier for this entity (e.g. snp_01BJQMF54D093DXEAWZ6JYRPAQ)",
                        max_length=30,
                        null=True,
                        prefix="bin_",
                        unique=True,
                    ),
                ),
                (
                    "created_at",
                    archivebox.abid_utils.models.AutoDateTimeField(
                        db_index=True, default=None
                    ),
                ),
                ("modified_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(default=None, max_length=63)),
                ("binprovider", models.CharField(default=None, max_length=31)),
                ("abspath", models.CharField(default=None, max_length=255)),
                ("version", models.CharField(default=None, max_length=32)),
                ("sha256", models.CharField(default=None, max_length=64)),
                (
                    "machine",
                    models.ForeignKey(
                        default=None,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="machine.machine",
                    ),
                ),
            ],
            options={
                "unique_together": {
                    ("machine", "name", "binprovider", "abspath", "version", "sha256")
                },
            },
        ),
    ]
