# Generated by Django 5.0.6 on 2024-08-20 01:54

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0044_alter_archiveresult_snapshot_alter_tag_uuid_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshot',
            name='old_id',
            field=models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True),
        ),
    ]
